/*
    Drools5 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools5;

/**
 * This enumeration define how parameter can be handled.
 * 
 * This parameter handling is related to interface.
 * @author mbo
 *
 */
public enum DroolsParameterHandling {
	/**
	 * This parameter is handle in normal way. 
	 * 
	 * In this case, the parameter is just copied into the working memory of the Drools
	 * engine.
	 */
	NONE,
	/**
	 * This parameter is ignored. 
	 * 
	 * This can be used for example when it is mandatory to receive a parameter, but this
	 * parameter is useless for drool procession.
	 */
	IGNORED,
	/**
	 * This parameter is an array or a list and requires flattering. 
	 * 
	 * All elements of the list/array must be pass to the drools engine and not the array directly.
	 */
	TOFLAT1,
	/**
	 * This parameter is an array or a list and requires recursive flattering.
	 * 
	 * All elements of the list/array must be pass to the drools engine, recursively and not
	 * the array directly.
	 */
	TOFLATALL,
	/**
	 * This parameter is a Class definition.
	 * 
	 * This parameter will be ignored for drools and used as the class to be used for the return
	 * value.
	 */
	GENERICDEFINITION
}
