/*
    Drools5 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools5;

import org.boretti.drools.integration.drools5.implementation.DroolsProviderImplementation;
import org.drools.RuleBase;

/**
 * This class provide a way to get a concrete proxy of an interface.
 * @author mbo
 *
 */
public class DroolsProvider {
	/**
	 * This is the method to be call to get the service.
	 * @param clazz The interface to be used to create the proxy
	 * @return The concrete instance (Proxy), based on the interface
	 */
	public <T> T getService(Class<T> clazz) {
		return DroolsProviderImplementation.getInstance().getService(clazz);
	}
	
	/**
	 * This is the method to be call to get the service.
	 * @param clazz The interface to be used to create the proxy
	 * @param resourceName override the resourceName annotation attribute
	 * @return The concrete instance (Proxy), based on the interface
	 * @since 1.2.0
	 */
	public <T> T getService(Class<T> clazz,String resourceName) {
		return DroolsProviderImplementation.getInstance().getService(clazz,resourceName);
	}
	
	/**
	 * This is the method to be call to get the service.
	 * @param clazz The interface to be used to create the proxy
	 * @param resourceName override the resourceName annotation attribute
	 * @param serviceType override the type annotation attribute
	 * @return The concrete instance (Proxy), based on the interface
	 * @since 1.2.0
	 */
	public <T> T getService(Class<T> clazz,String resourceName,DroolsServiceType serviceType) {
		return DroolsProviderImplementation.getInstance().getService(clazz,resourceName,serviceType);
	}
	
	/**
	 * This is the method to get a rule base from an annotated class.
	 * @param clazz The class that is instrumented and need a rulebase.
	 * @return the rule base
	 * @since 1.0.0
	 */
	public <T> RuleBase getRuleBase(Class<T> clazz) {
		return DroolsProviderImplementation.getInstance().getRuleBase(clazz);
	}
	
	/**
	 * This is the method to get a rule base from an annotated class.
	 * @param clazz The class that is instrumented and need a rulebase.
	 * @param resourceName override the resourceName annotation attribute.
	 * @return the rule base
	 * @since 1.2.0
	 */
	public <T> RuleBase getRuleBaseOverride(Class<T> clazz,String resourceName) {
		return DroolsProviderImplementation.getInstance().getRuleBaseOverride(clazz,resourceName);
	}
	
	/**
	 * This is the method to get a rule base from an annotated class.
	 * @param clazz The class that is instrumented and need a rulebase.
	 * @param resourceType override the type annotation attribute.
	 * @return the rule base
	 * @since 1.2.0
	 */
	public <T> RuleBase getRuleBaseOverride(Class<T> clazz,DroolsServiceType resourceType) {
		return DroolsProviderImplementation.getInstance().getRuleBaseOverride(clazz,resourceType);
	}
	
	/**
	 * This is the method to get a rule base from an annotated class.
	 * @param clazz The class that is instrumented and need a rulebase.
	 * @param resourceName override the resourceName annotation attribute.
	 * @param resourceType override the type annotation attribute.
	 * @return the rule base
	 * @since 1.2.0
	 */
	public <T> RuleBase getRuleBaseOverride(Class<T> clazz,String resourceName,DroolsServiceType resourceType) {
		return DroolsProviderImplementation.getInstance().getRuleBaseOverride(clazz,resourceName,resourceType);
	}
	
	/**
	 * This is the method to get a rule base from a type and a resourceName.
	 * 
	 * RuleBase are cached.
	 * 
	 * @param clazz The class that is instrumented and need a rulebase.
	 * @param type The type to be used.
	 * @param resourceName The resource name to be used.
	 * @return the rule base
	 * @since 1.1.0
	 */
	public <T> RuleBase getRuleBase(Class<T> clazz,DroolsServiceType type,String resourceName) {
		return DroolsProviderImplementation.getInstance().getRuleBase(clazz, type, resourceName);
	}
	
	/**
	 * This is the method that is call to evaluate pre-condition.
	 * 
	 * @param clazz The class that is instrumented and need evaluation.
	 * @param type The type of the drools resource.
	 * @param resourceName The name of the drools resource.
	 * @param error The error class to be used for user exception detection.
	 * @param reference The object that the request is from.
	 * @param args The parameter of the calling method.
	 * @since 1.1.0
	 */
	public static <T> void runPreCondition(Class<T> clazz,
									DroolsServiceType type,
									String resourceName,
									Class<? extends Error> error,
									Object reference,
									Object args[]) {
		DroolsProviderImplementation.getInstance().runPreCondition(clazz, type, resourceName, error, reference, args);
	}
	
	/**
	 * This is the method that is call to evaluate post-condition.
	 * 
	 * @param clazz The class that is instrumented and need evaluation.
	 * @param type The type of the drools resource.
	 * @param resourceName The name of the drools resource.
	 * @param error The error class to be used for user exception detection.
	 * @param reference The object that the request is from.
	 * @param args The parameter of the calling method.
	 * @since 1.1.0
	 */
	public static <T> void runPostCondition(Class<T> clazz,
									DroolsServiceType type,
									String resourceName,
									Class<? extends Error> error,
									Object reference,
									Object args[]) {
		DroolsProviderImplementation.getInstance().runPostCondition(clazz, type, resourceName, error, reference, args);
	}
	
}
