/*
    Drools5 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools5.implementation;

import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.util.List;
import java.util.concurrent.Callable;

import org.apache.log4j.Logger;
import org.boretti.drools.integration.drools5.DroolsSessionType;

/**
 * @author mbo
 *
 */
class DroolsCallable<T> implements Callable<Object> {
	
	private final Logger logger = Logger.getLogger(DroolsCallable.class);

	private DroolsProxy<T> dp;
	
	private Object proxy;
	
	private Method method;
			
	private DroolsSessionType session;
	
	private Object[] args;
	
	private List<Object> fact;
	
	private Class<?> returnType;
	
	private String activationGroup;
	
	public DroolsCallable(DroolsProxy<T> dp,Object proxy,Class<?> returnType,Method method,DroolsSessionType session,Object[] args,List<Object> fact,String activationGroup) {
		this.dp=dp;
		this.proxy=proxy;
		this.method=method;
		this.session=session;
		this.args=args;
		this.fact=fact;
		this.returnType=returnType;
		this.activationGroup=activationGroup;
	}
	
	@Override
	public Object call() throws Exception {
		try {
			Class<?> rt = returnType;
			if (rt==null) rt=(Class<?>)((ParameterizedType)method.getGenericReturnType()).getActualTypeArguments()[0];
			return dp.invokeInternal(proxy,method,rt,session,args,fact,activationGroup);
		} catch (Throwable e) {
			logger.error(""+e.getMessage(),e);
			throw new Exception(""+e.getMessage(),e);
		}
	}

}
