/*
    Drools5 Integration Helper
    Copyright (C) 2009  Mathieu Boretti mathieu.boretti@gmail.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */
package org.boretti.drools.integration.drools5.implementation;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.log4j.Logger;
import org.boretti.drools.integration.drools5.DroolsParameterHandling;
import org.boretti.drools.integration.drools5.annotations.DroolsParameter;
import org.boretti.drools.integration.drools5.exceptions.IncompatibleAnnotationDroolsError;

class DroolsProductFactResult {

	private static final Logger logger = Logger
			.getLogger(DroolsProductFactResult.class);

	DroolsProductFactResult(Method m, Object[] args) {
		if (args == null)
			return;
		if (args.length == 0)
			return;
		Annotation[][] ass = m.getParameterAnnotations();
		for (int i = 0; i < args.length; i++) {
			Annotation[] as = ass[i];
			DroolsParameterHandling handling = DroolsParameterHandling.NONE;
			for (Annotation a : as) {
				if (a.annotationType().equals(DroolsParameter.class)) {
					handling = ((DroolsParameter) a).handling();
					break;
				}
			}
			if (handling == null)
				handling = DroolsParameterHandling.NONE;
			if (DroolsParameterHandling.NONE.equals(handling)) {
				getFact().add(args[i]);
			} else if (DroolsParameterHandling.IGNORED.equals(handling)) {
			} else if (DroolsParameterHandling.GENERICDEFINITION
					.equals(handling)) {
				if (args[i] instanceof Class) {
					//null check
					setReturnType((Class<?>) args[i]);
				}else {
					String msg = "GenericDefinition can only be used on Class parameter";
					logger.error(msg);
					throw new IncompatibleAnnotationDroolsError(msg);
				}
			} else if (DroolsParameterHandling.TOFLAT1.equals(handling)) {
				if (args[i] instanceof Object[])
					for (Object o : (Object[]) args[i])
						getFact().add(o);
				else if (args[i] instanceof int[])
					for (int o : (int[]) args[i])
						getFact().add(o);
				else if (args[i] instanceof long[])
					for (long o : (long[]) args[i])
						getFact().add(o);
				else if (args[i] instanceof short[])
					for (short o : (short[]) args[i])
						getFact().add(o);
				else if (args[i] instanceof byte[])
					for (byte o : (byte[]) args[i])
						getFact().add(o);
				else if (args[i] instanceof double[])
					for (double o : (double[]) args[i])
						getFact().add(o);
				else if (args[i] instanceof float[])
					for (float o : (float[]) args[i])
						getFact().add(o);
				else if (args[i] instanceof boolean[])
					for (boolean o : (boolean[]) args[i])
						getFact().add(o);
				else if (args[i] instanceof Collection)
					for (Object o : (Collection<?>) args[i])
						getFact().add(o);
				else
					getFact().add(args[i]);
			} else if (DroolsParameterHandling.TOFLATALL.equals(handling)) {
				addFlat(getFact(), args[i]);
			}
		}
	}

	private static void addFlat(List<Object> fact, Object args) {
		if (args==null) return;
		if (args instanceof Object[])
			for (Object o : (Object[]) args)
				addFlat(fact, o);
		else if (args instanceof int[])
			for (int o : (int[]) args)
				addFlat(fact, o);
		else if (args instanceof long[])
			for (long o : (long[]) args)
				addFlat(fact, o);
		else if (args instanceof short[])
			for (short o : (short[]) args)
				addFlat(fact, o);
		else if (args instanceof byte[])
			for (byte o : (byte[]) args)
				addFlat(fact, o);
		else if (args instanceof double[])
			for (double o : (double[]) args)
				addFlat(fact, o);
		else if (args instanceof float[])
			for (float o : (float[]) args)
				addFlat(fact, o);
		else if (args instanceof boolean[])
			for (boolean o : (boolean[]) args)
				addFlat(fact, o);
		else if (args instanceof Collection)
			for (Object o : (Collection<?>) args)
				addFlat(fact, o);
		else
			fact.add(args);
	}

	private List<Object> fact = new ArrayList<Object>();
	private Class<?> returnType = null;

	/**
	 * @return the fact
	 */
	List<Object> getFact() {
		return fact;
	}

	/**
	 * @param fact
	 *            the fact to set
	 */
	void setFact(List<Object> fact) {
		this.fact = fact;
	}

	/**
	 * @return the returnType
	 */
	Class<?> getReturnType() {
		return returnType;
	}

	/**
	 * @param returnType
	 *            the returnType to set
	 */
	void setReturnType(Class<?> returnType) {
		this.returnType = returnType;
	}
}
